<?php
/**
 * @package Plugin ChjoomLivesession
 * @version 1.0.4
 * @author Cloudhoreca Kft.
 * @copyright (C) 2022 - Cloudhoreca Kft.
 * @license GNU/GPLv2 https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
**/


use Joomla\CMS\Factory;

defined('_JEXEC') or die();
jimport('joomla.plugin.plugin');

class plgSystemChjoomLivesession extends JPlugin
{
    /**
     * Constructor.
     *
     * @param object $subject The object to observe
     * @param array $config An optional associative array of configuration settings
     */
    public function __construct(&$subject, array $config)
    {
        // Calling the parent Constructor
        parent::__construct($subject, $config);
    }

    function onAfterDispatch()
    {
        if (Factory::getApplication()->isClient('site'))
        {
            $input = Factory::getApplication()->input;
            $option = $input->get('option');
            $view = $input->get('view');
            $task = $input->get('task');
            $format = $input->get('format');
            $itemId = $input->get('Itemid');
            $user = Factory::getUser();

            $userId = $user->get('id', 0);
            $followJourney = $this->params->get('follow_journey');

            if ($option === 'com_content' && $view === 'category')
            {
                $catId = $input->get('id', '0', 'string' );
            }
            else
            {
                if (JFactory::getConfig()->get('sef'))
                {
                    $articleId = $input->get('id');
                    $catId = $this->getCategory($articleId);
                }
                else
                {
                    $catId = $input->get('catid', '0', 'string' );
                }
            }

            $selectionMethods = [
                'component' => $this->params->get('component_record_type', '-1'),
                'category'  => $this->params->get('category_record_type', '-1'),
                'menuitem'  => $this->params->get('menuitem_record_type', '-1')
            ];

            $assign = [
                'component' => $this->params->get('components', array()),
                'category'  => $this->params->get('categories', array()),
                'menuitem'  => $this->params->get('menuitems', array())
            ];

            if ($selectionMethods['component'] === '-1')
            {
                $isComponent = false;
            }
            else
            {
                $isComponent = $selectionMethods['component'] === '0' ? !in_array($option, $assign['component']) : in_array($option, $assign['component']);
            }

            if ($selectionMethods['category'] === '-1')
            {
                $isCategory = false;
            }
            else
            {
                $isCategory = $selectionMethods['category'] === '0' ? !in_array($catId, $assign['category']) : in_array($catId, $assign['category']);
            }

            if ($selectionMethods['menuitem'] === '-1')
            {
                $isMenuItem = false;
            }
            else
            {
                $isMenuItem = $selectionMethods['menuitem'] === '0' ? !in_array($itemId, $assign['menuitem']) : in_array($itemId, $assign['menuitem']);
            }

            $trackPage = $isCategory || $isComponent  || $isMenuItem ;

            if (in_array('0', array_values($selectionMethods)))
            {
                foreach ($selectionMethods as $type => $method)
                {
                    if ($method === '0')
                    {
                        switch ($type)
                        {
                            case 'category':
                                if (in_array($catId, $assign['category']))
                                {
                                    $trackPage = false;
                                }
                                break;

                            case 'component':
                                if (in_array($option, $assign['component']))
                                {
                                    $trackPage = false;
                                }
                                break;

                            case 'menuitem':
                                if (in_array($itemId, $assign['menuitem']))
                                {
                                    $trackPage = false;
                                }
                                break;
                        }
                    }
                }
            }

            // if already visited a tracked page, and follow journey is enabled, we have to record all the pages
            $session = Factory::getSession();
            if ($followJourney && $session->get('lvs_follow'))
            {
                $trackPage = true;
            }

            if ($trackPage)
            {
                $customParams = $this->params->get('custom_params', array());

                if ($this->params->get('senduserinfo') === '1')
                {
                    $customParamVariables = array();
                    foreach ((array) $customParams as $customParam)
                    {
                        if ($customParam->variable === 'user_id')
                        {
                            $sessionVariable = $userId;
                        }
                        else
                        {
                            $sessionVariable = $session->get($customParam->variable) ?: $input->get($customParam->variable);
                        }
                        if ($sessionVariable)
                        {
                            $customParamVariables[] = $customParam->variable . ": '" . $sessionVariable . "'";
                        }
                    }
                    $paramsString = implode(',', $customParamVariables);
                    $identify = "__ls('identify', { name: '" . $user->get('name') . "', email: '" . $user->get('email') . "', params: { ".$paramsString . " } });";
                }

                $js = trim($this->params->get('trackingcode'));
                $js .= (!empty($identify) ? $identify : '');
                Factory::getDocument()->addScriptDeclaration($js);

                if ($followJourney)
                {
                    $session->set('lvs_follow', true);
                }
            }
        }
    }

    private function getCategory($articleId)
    {
        if (empty($articleId)) return '';
        $db = Factory::getDbo();
        $query = $db->getQuery(true)
            ->select('catid')
            ->from($db->quoteName('#__content'))
            ->where($db->quoteName('id') . '=' . $db->quote($articleId));
        return $db->setQuery($query)->loadResult();
    }
}